<?php
/**
 * Class to implement an online commerce application in WordPress using the Mal's Cart service
 *
 * This class is the base class for the plugin and it's main task is to register the settings and set all the hooks
 *
 * PHP version 5.1.2
 *
 *
 * @package			WP_Malscart
 * @author			Roland Barker <webdesign@xnau.com>
 * @copyright		2011 xnau webdesign
 * @version			0.5.5
 * @license			GPL2
 * @link				http://xnau.com/wp-malscart
 * @since			File available since Release 0.1
 */

class WP_Malscart 
{

	// slug identifer
	const PLUGIN_NAME = 'wp-malscart';
	
	// print title for the plugin
	const PLUGIN_TITLE = "WP Mal's Cart";
	
	// current software revision
	const PLUGIN_VERSION = '0.5.5';
	
	// Mal's Cart shopping cart domain for posting; we use this to error-check user input
	public static $malscart_cart_domain = '.aitsafe.com';
	
	// slug for the settings page
	public static $OPTIONS_PAGE_NAME;
	
	// name of the plugin options field
	public static $OPTIONS_SETTINGS;
	
	// absolute url for the plugin's directory
	public static $PLUGIN_URL;
	
	// realtive path to the plugin's directory
	public static $PLUGIN_PATH;
	
	// true if the plugin's options page is displaying
	public static $ON_OPTIONS_PAGE;
	
	// true after settings have been updated
	public static $is_update;
	
	// this array holds the settings tabs as slug/title pairs
	public static $sections = array();
	
	// this array holds the default plugin settings
	public static $setting_defaults;
	
	// array holds the default parameters for a product purchase button
	public static $product_form_fields;
	
	// this boolean flags the presence of a plugin shortcode in the content (not working reliably yet)
	public $shortcode_present = false;

	public function __construct() {

		self::$PLUGIN_URL = WP_PLUGIN_URL . "/" . self::PLUGIN_NAME;
		self::$PLUGIN_PATH = WP_PLUGIN_DIR . "/" . self::PLUGIN_NAME;
		self::$OPTIONS_PAGE_NAME = self::PLUGIN_NAME;
		self::$OPTIONS_SETTINGS = self::PLUGIN_NAME.'-settings';
		self::$ON_OPTIONS_PAGE = ( isset( $_GET['page'] ) && $_GET[ 'page' ] == 'wp-malscart' ) ? true : false;
		self::$is_update = ( isset($_GET['settings-updated']) && $_GET['settings-updated'] == 'true' ) ? true : false;

		$this->plugin_settings = array();
		$this->load_settings();
		
		// if the setting is not present, initialize it
		if ( ! get_option( self::$OPTIONS_SETTINGS ) )
			$this->initialize_settings();

		// define the settings sections
		// these will end up on tabs
		$this->sections = array(
														'main_options' => 'Main Settings',
														'buy_button_options' => 'Buy Button Preferences',
														'help' => 'Help'
														);
		
		// buy button shortcode attributes		
		// this is where we would add any extra fields like product id numbers or state hashes
		// array is filled with default values that can be overridden
		self::$product_form_fields = array( 
												'button_style'		=> self::get_option( 'button_style' ),
												'product[]'			=> self::get_option( 'test_product' ),
												'price'				=> self::get_option( 'test_price' ),
												'return'				=> self::get_option( 'return' ),
												'button_text'		=> self::get_option( 'button_text' ),
												'button_width'				=>	self::get_option( 'button_width' )
												);

		// default values for a plugin setting
		self::$setting_defaults = array(
												'id'			=> 'default_field',
												'title'		=> 'Default Field',
												'desc'		=> 'This is a default description.',
												'value'		=> '',
												'type'		=> 'text',
												'section'	=> 'main_options',
												'items'		=> array(), // this will be a list of options
												'class'		=> ''
		);
		
		add_action( 'admin_init', array( $this, 'admin_init' ) );
		
		add_action( 'admin_menu', array( 'WP_Malscart_Options_Page', 'initialize' ) );
		
		// settings validation faults and user feeback in the admin
		add_action( 'admin_notices', array( 'WP_Malscart_Options_Page', 'display_errors' ), 15 );
		
		//add_filter('the_posts', array( $this, 'shortcode_check' ) );// this is problematic and unnecessary

		// add the button CSS
		add_action( 'wp_print_styles', array( $this, 'add_plugin_CSS' ) );

		// admin steeings page CSS
		add_action( 'admin_print_styles', array( 'WP_Malscart_Options_Page', 'admin_header' ) );
		
		// shortcode to get the plugin's info; used on help pages
		add_shortcode( 'wp_malscart_info', array( 'WP_Malscart', 'get_plugin_info' ) );

		// register the shortcode
		add_shortcode( 'buy_button', array( 'WP_Malscart_HTML', 'buy_button' ) );
		
		// internationalization
		load_plugin_textdomain( self::PLUGIN_NAME, false, basename( dirname( __FILE__ ) ) . '/languages' );
		
	}
	
	/*
	
	this function lays out all the settings fields
	and their sections
	
	
	*/
	public function admin_init() {
		
		$this->register_settings();
	
	}


	/**
	* defines all settings for the plugin
	*
	* each statement defines one setting by populating the $plugin_settings array
	*
	* @return NULL
	*
	*/
	private function load_settings() {

		$this->plugin_settings['cart_server'] = array( 
																									
			'title'			=> __('Cart Server', self::PLUGIN_NAME ), 
			'desc'			=> __('The server name for your cart; something like "http://ww1.aitsafe.com"', self::PLUGIN_NAME), 
			'type'			=> 'text',
			'value'			=> '',
			'section'		=> 'main_options'
			
		);

		$this->plugin_settings['cust_id'] = array( 
																							
			'title'			=> __( "Mal's Cart Username", self::PLUGIN_NAME ), 
			'desc'			=> __( 'This is your 8-digit username', self::PLUGIN_NAME ), 
			'type'			=> 'text',
			'value'			=> '',
			'section'		=> 'main_options'
			
		);
		/*  the shortcode provides this itself; we leave this here in case we need it later
		$this->plugin_settings['return'] = array( 
																						 
			'title'			=> __("Return URL", self::PLUGIN_NAME ), 
			'desc'			=> __('This is the fallback address for "return to shopping" buttons on cart pages', self::PLUGIN_NAME ), 
			'type'			=> 'text',
			'value'			=> get_bloginfo( 'url' ),
			'section'		=> 'main_options'
			
		);
		*/
		$this->plugin_settings['button_CSS'] = array( 
																								 
			'title'			=> __("Use plugin CSS?", self::PLUGIN_NAME ), 
			'desc'			=> '', 
			'type'			=> 'radio',
			'value'			=> 1,
			'section'		=> 'buy_button_options', 
			'items' 			=> array( 
									__('Yes (plugin CSS will style buy buttons)', self::PLUGIN_NAME ) => 1, 
									__('No (buttons will be styled by theme CSS)', self::PLUGIN_NAME ) => 0
								)
			
		);

		$this->plugin_settings['button_width'] = array( 
																									
			'title'			=> __('Width', self::PLUGIN_NAME ), 
			'desc'			=> __('the default width of the box containing the button', self::PLUGIN_NAME ), 
			'type'			=> 'text',
			'value'			=> '150',
			'section'		=> 'buy_button_options'
		);

		$this->plugin_settings['selector_preference'] = array( 
																													
			'title'			=> __("Which selector for product options?", self::PLUGIN_NAME ), 
			'desc'			=> '', 
			'type'			=> 'radio',
			'value'			=> 'radio',
			'section'		=> 'buy_button_options', 
			'items' 			=> array( 
									__('Radio Buttons (such as these)', self::PLUGIN_NAME ) => 'radio', 
									__('Pull-down List (best for a large number of choices)', self::PLUGIN_NAME ) => 'select'
								)
			
		);

		$this->plugin_settings['button_text'] = array( 
																									
			'title'			=> __('Buy Button Label', self::PLUGIN_NAME ), 
			'desc'			=> __('the default text that will appear on buy buttons', self::PLUGIN_NAME ), 
			'type'			=> 'text',
			'value'			=> 'Add to Cart',
			'section'		=> 'buy_button_options',
			'class'			=> 'special-text'
		);
		

		$this->plugin_settings['test_product'] = array( 
																									 
			'title'			=> __("Test Product Name", self::PLUGIN_NAME ), 
			'desc'			=> '', 
			'type'			=> 'text',
			'value'			=> 'Test Product',
			'section'		=> 'buy_button_options'
		);
		

		$this->plugin_settings['test_price'] = array(
																								 
			'title'			=> __('Test Product Price', self::PLUGIN_NAME ), 
			'desc'			=> '', 
			'type'			=> 'text',
			'value'			=> '25.00',
			'section'		=> 'buy_button_options'
			
		);

		$this->plugin_settings['button_style'] = array( 
																									 
			'title'			=> __('Select Button Style', self::PLUGIN_NAME ), 
			'desc'			=> __('Select a base style for the buy button shorcode', self::PLUGIN_NAME ), 
			'type'			=> 'button_style',
			'value'			=> 1,
			'section'		=> 'buy_button_options', 
			'items' 			=> array( 
									__('Simple', self::PLUGIN_NAME ) => 1,
									__('Product and price', self::PLUGIN_NAME ) => 2
								)
			
		);



	}	

	/**
	* Register settings with the WP Settings API
	*
	* register the setting array with WP, then add the sections, then all the settigns
	*/
	private function register_settings() {
		
		register_setting( 
										 self::$OPTIONS_SETTINGS, 
										 self::$OPTIONS_SETTINGS, 
										 array( $this,'options_validate' ) 
										 );

		foreach ( $this->sections as $slug => $title )
			add_settings_section( 
												 $slug, 
												 $title, 
												 array( 'WP_Malscart_Options_Page', 'print_section' ), 
												 self::$OPTIONS_PAGE_NAME 
												 );

		foreach ( $this->plugin_settings as $id => $setting ) {
			$setting['id'] = $id;
			$this->create_setting( $setting );
		}

	}

	/**
	* add the setting field to the options page
	*
	* merge the setting attributes with the defaults before submitting the settings 
	* attributes to the "display_setting_field" method
	*
	* @param $args array
	*		type	the input type for the setting
	*		id		the id slug of the setting
	*		desc	help text for the input field
	*		value	the current value of the setting
	*		items	array containing the items for radio buttons or options elements
	*		class	CSS class for the element
	*
	*/
	private function create_setting( $args = array() ) {
		
		$settings = wp_parse_args( $args, self::$setting_defaults );
		extract( $settings );

		// all the arguments we're going to pass to each setting field
		$field_args = array(
				'type'      => $type,
				'id'        => $id,
				'desc'      => $desc,
				'value'       => $value,
				'items'		=> $items,
				'label_for' => $id,
				'class'     => $class
		);

		add_settings_field( $id, $title, array( 'WP_Malscart_Options_Page','display_setting_field' ), self::$OPTIONS_PAGE_NAME, $section, $field_args );

	}

	
	/**
	* validate settings on submit
	*
	* user settings submissions are verfied before the settings update can be accepted
	* a long swtich statement filters out some values for verification
	* 
	* @param $settings array
	* 	each setting id is the key
	* 
	*/
	public function options_validate( Array $settings ) {
		
		//error_log( __METHOD__.' validating' );
		
		
		foreach( $settings as $key => $value ) :

			switch ( $key ) {
				
				case 'cart_server' :
				
					// this breaks the value into pieces and keeps the important part
					$regex = '|^(?P<protocol>http(?P<secure_flag>s)?://)(?P<subdomain>[a-z0-9-]{3})\.'.ltrim( self::$malscart_cart_domain, '.' ).'(?P<path>/.*)?$|';
					
					$value = strtolower( $value );
					
					if ( empty( $value ) ) 
						add_settings_error( 'Cart Server', 'cart_server', __('The Cart URL is required', self::PLUGIN_NAME ) );
					
					elseif ( ! preg_match( $regex, $value, $matches ) )
						add_settings_error( 'Cart Server', 'cart_server', __('Check that the Cart URL is complete and correct', self::PLUGIN_NAME ) );
					
					else $settings[ 'cart_server' ] = $matches[ 'protocol' ].$matches[ 'subdomain' ].self::$malscart_cart_domain;
					
					break;
					
				case 'cust_id' :
				
					if ( empty( $value ) ) 
						add_settings_error( "Mal's Cart Username", 'cust_id', __("Your Mal's Cart Username is required", self::PLUGIN_NAME ) );
						
					elseif ( ! preg_match( '|^([0-9]{8})$|', $value ) ) // we need an 8-digit number
						add_settings_error( "Mal's Cart Username", 'cust_id', __("Your Mal's Cart Username seems to be incorrect", self::PLUGIN_NAME ) );
						
					break;

				case 'button_width' :
			
					$regex = "#([0-9\.]+)(px|em|pt|%|ex)?#";	// grab only valid CSS values
					if ( ! $valid = preg_match( $regex, $value, $matches ) ) add_settings_error( 'Width', 'button_width', __( 'Button width needs to be a number (for pixels) or a number with a CSS unit', self::PLUGIN_NAME ) );
					$settings[ 'button_width' ] = $matches[1] . ( empty( $matches[2] ) ? 'px' : $matches[2] );

					break; 	
					
				case 'return':// not used
				
					$regex = "|^http(s)?\://[a-zA-Z0-9\-\.]+\.[a-zA-Z]{2,3}(/\S*)?$|";
					if ( empty( $value ) )
						add_settings_error( "Return URL", 'return', __("A return URL is required", self::PLUGIN_NAME ) );
						
					elseif ( ! preg_match( $regex, $value ) )
						add_settings_error( "Return URL", 'return', __("A valid return URL is required", self::PLUGIN_NAME ) );
					
				default :
				
			}
		endforeach;
					
		return $settings;
		
	}
	
	
	// easy way to grab one value or see if it is set
	public function get_option( $key ) {
		
		$settings = get_option( self::$OPTIONS_SETTINGS );
		
		return empty( $settings[ $key ] ) ? false : $settings[ $key ];
		
	}
	/*	
	 *
	 * Set Default Settings Values
	 *
	 * this triggers on first activation to fill the options record in the db
	 *
	 */
	public function initialize_settings() {
	
		$default_settings = array();
		foreach ( $this->plugin_settings as $id => $setting ) {
			if ( $setting['type'] != 'heading' )
				$default_settings[$id] = $setting['value'];
		}
	
		update_option( self::$OPTIONS_SETTINGS, $default_settings );
	
	}

	// provides a way to check the content for one of our shortcodes
	// for now, we don't use this, it needs fixing and we don't need it
	// but we may eventually need it...
	// this is meant to work with the `the_posts` filter
	public function shortcode_check( $post ) {
		
		if (empty($posts)) return $posts;

		foreach ($posts as $post) {
			if ( false !== stripos( $post->post_content, '[buy_button' ) ) {
				$this->shortcode_present = true;
				break;
			}
		}
	 
		return $posts;
	}
	
	// return info about the plugin	
	public function plugin_data() {
		
		if ( function_exists( 'get_plugin_data' ) )
			return get_plugin_data( WP_MALSCART_PLUGIN_FILE );
		else 
			return array( 'Version'=>self::PLUGIN_VERSION );
		
	}
	
	public function get_plugin_info( $atts ) {
		
		extract( shortcode_atts( array( 'field' => 'Version' ), $atts ) );
		
		$info = self::plugin_data();
		
		return $info[ $field ];
		
	}
	
	// conditionally adds the button CSS
	public function add_plugin_CSS() {

		if ( $this->get_option( 'button_CSS' ) )
		
		wp_register_style( self::PLUGIN_NAME.'-css', self::$PLUGIN_URL.'/'.self::PLUGIN_NAME.'-buttons.css' );
		
		wp_enqueue_style( self::PLUGIN_NAME.'-css' );
		
	}
																 
	

} // class

/*  
		Copyright 2011  Roland Barker  (email : webdesign@xnau.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/?>
